# -*- coding: utf-8 -*-
import argparse
import os
import re
import sys
import subprocess
import time

# - uruchamianie testów z pokryciem kodu
# - zapis wyniku testów do logu
# - jeżeli testy powiodły się - na zielono wyświetl ilość uruchomionych testów
# i poziom pokrycia kodu
# - jeżeli wystąpił błąd w testach - na pomarańczowo wyświetl ilość
# uruchomionych testów i poziom pokrycia kodu
# - jeżeli testy nie przeszły - na czerwono wyświetl ilość uruchomionych testów
# i poziom pokrycia kodu
# - jeżeli jakiś plik zmienił się, uruchom ponownie test
DEFAULT_COLORS = {
    'red': 41,
    'green': 42,
    'blue': 44,
    'sea': 46,
}


class StatusDisplay(object):
    """ <<sink>>

    Responsibilities:

        - print status information
    """
    pattern = '%(color)s %(num)8d%(failures)9d%(errors)8d%(coverage_color)s%(coverage)8d%%%(normal_color)s\n'
    empty_pattern = '%(color)s%(space)28s%(coverage_color)s%(space)8s%%%(normal_color)s\n'

    def __init__(self, output=None, colors=None):
        """@todo: Docstring for __init__

        :param output: file to which write
        :param colors: color codes used to display status
        """
        self._output = output if output is not None else sys.stdout
        self._colors = colors if colors is not None else DEFAULT_COLORS
        self._last_run_color = ''
        self._last_run_cov_color = ''

    def write_header(self):
        self._output.write('Tests ran Failures  Errors Coverage\n')

    def write_last(self):
        self._output.write('\n')

    def write(self, ran, failures, errors, coverage):
        """ Display status information.

        :param ran: number of tests run
        :param failures: number of failures
        :param errors: number of errors
        :param coverage: coverage percent
        """
        if failures or errors:
            color = self._colors['red']
            coverage_color = color
        else:
            green = self._colors['green']
            blue = self._colors['blue']
            if self._last_run_color in [green, blue]:
                color = blue
            else:
                color = green
            if coverage < 100:
                coverage_color = self._colors['sea']
            else:
                coverage_color = color
        self._last_run_color = color
        self._last_run_cov_color = coverage_color
        line = self.pattern % {
            'color': '\033[%sm' % color,
            'num': ran,
            'failures': failures,
            'errors': errors,
            'coverage_color': '\033[%sm' % coverage_color if coverage_color != color else '',
            'coverage': coverage,
            'normal_color': '\033[0m',
        }
        self._output.write(line)


class AbstractWriter(object):

    def write(self, text):
        raise NotImplementedError  # pragma nocover


class DummyWriter(AbstractWriter):

    def write(self, text):
        pass


class LogWriter(AbstractWriter):
    """ <<sink>>

    Responsibilities:

        - write logs
    """

    def __init__(self, output):
        """@todo: Docstring for __init__

        :param output: @todo
        :returns: @todo

        """
        self._output = output
        self._last_traceback = []

    def write(self, text):
        """@todo: Docstring for write

        :param text: @todo
        :returns: @todo

        """
        to_diff = text.splitlines()
        to_diff = [line for line in to_diff if not line.startswith('Ran')]
        if to_diff != self._last_traceback:
            self._output.write(text)
            self._output.flush()
            self._last_traceback = to_diff


class TestRunner(object):
    """ <<source>>

    Responsibilities:

        - run tests
    """
    def __init__(self, command):
        """@todo: Docstring for __init__

        :param command: @todo
        :returns: @todo

        """
        self._command = command

    def run(self):
        """@todo: Docstring for run
        :returns: @todo
        """
        command = [
            'coverage', 'run',
            '--branch',
            '--source', '.',
        ] + self._command.split()
        program_output, test_output = self._run_command(command)
        command = ['coverage', 'report']
        report_output, error_output = self._run_command(command)
        return (str(program_output), str(test_output + report_output))

    def _run_command(self, command):
        process = subprocess.Popen(command, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        stdoutdata, stderrdata = process.communicate()
        return stdoutdata, stderrdata


class TestResultParser(object):
    """ <<filter>>

    Responsibilities:

        - parse test result
    """

    def parse(self, input_data):
        lines = input_data.splitlines()
        lines.reverse()
        ran = self._get_ran_tests_num(lines)
        failures, errors = self._get_failures(lines)
        coverage = self._get_coverage(lines)
        return (ran, failures, errors, coverage)

    def _get_failures(self, lines):
        error_re = re.compile('^(?:OK|FAILED)(?:\s*\((?:failures=(\d+))?(?:,\s*)?(?:errors=(\d+))?)')
        failures = 0
        errors = 0
        for line in lines:
            match = error_re.search(line)
            if match:
                failures = int(match.group(1) or 0)
                errors = int(match.group(2) or 0)
                return failures, errors
        return failures, errors

    def _get_ran_tests_num(self, lines):
        ran_re = re.compile('^Ran (\d+)')
        ran = 0
        for line in lines:
            match = ran_re.search(line)
            if match:
                return int(match.group(1) or 0)
        return ran

    def _get_coverage(self, lines):
        total_re = re.compile('^[^\s]+\s*\d+\s*\d+\s*\d+\s*\d+\s*(\d+)%')
        total = 0
        for line in lines:
            match = total_re.search(line)
            if match:
                return int(match.group(1) or 0)
        return total


class FileMonitorTimeoutError(Exception):
    pass


class FileMonitor(object):
    """ <<source>>

    Responsibilities:

        - monitor file changes
    """
    def __init__(self, timeout=60, interval=2, monitored_extensions=['.py']):
        self._mtimes = {}
        self._timeout = timeout
        self._interval = interval
        self._monitored_extensions = monitored_extensions

    def wait_for_change(self):
        """@todo: Docstring for wait_for_change
        :returns: @todo

        """
        time_passed = 0
        while not self.code_has_changed():
            time.sleep(self._interval)
            time_passed += self._interval
            if time_passed > self._timeout:
                raise FileMonitorTimeoutError
        return True

    def code_has_changed(self):
        files = self.get_monitored_files()
        for filename in files:
            if self.file_has_changed(filename):
                return True
        return False

    def get_monitored_files(self):
        for dirpath, dirnames, filenames in os.walk('.'):
            for filename in filenames:
                if self.should_monitor_file(filename):
                    yield os.path.join(dirpath, filename)

    def should_monitor_file(self, filename):
        file_extension = os.path.splitext(filename)[1]
        return file_extension in self._monitored_extensions

    def file_has_changed(self, filename):
        curr_mtime = os.stat(filename).st_mtime
        last_mtime = self._mtimes.setdefault(filename, curr_mtime)
        self._mtimes[filename] = curr_mtime
        return (curr_mtime != last_mtime)


class TddMon(object):
    """ <<controller>>

    Collaborators:

        - TestRunner - <<source>>
        - TestResultParser - <<filter>>
        - LogWriter - <<sink>>
        - StatusDisplay - <<sink>>
        - FileMonitor - <<source>>
    """
    def __init__(self, filename, log=None, output=None):
        self.test_runner = TestRunner(filename)
        self.log_writer = LogWriter(log) if log is not None else DummyWriter()
        self.test_result_parser = TestResultParser()
        self.status_display = StatusDisplay(output)
        self.file_monitor = FileMonitor()

    def run(self):
        """@todo: Docstring for run
        :returns: @todo

        """
        stdoutdata, stderrdata = self.test_runner.run()
        self.log_writer.write(stdoutdata + stderrdata)
        result = self.test_result_parser.parse(stderrdata)
        self.status_display.write(*result)

    def loop(self):
        self.status_display.write_header()
        self.run()
        try:
            while True:
                try:
                    self.file_monitor.wait_for_change()
                except FileMonitorTimeoutError:
                    self.status_display.write_last()
                else:
                    self.run()
        except KeyboardInterrupt:
            pass


def main(*args):
    """ Run test monitor
    """
    parser = argparse.ArgumentParser()
    parser.add_argument('filename')
    parser.add_argument('-l', '--log', dest='log', type=argparse.FileType('a'))
    result = parser.parse_args(*args)
    controller = TddMon(result.filename, log=result.log)
    controller.loop()


if __name__ == '__main__':  # pragma nobranch
    main()  # pragma nocover



# -*- coding: utf-8 -*-
import re
import unittest

from mock import patch
from morelia import Parser
from six.moves import StringIO
from smarttest.decorators import test_type

from tddmon import DEFAULT_COLORS, TddMon


@test_type('acceptance')
class RunMonitorTestCase(unittest.TestCase):
    " Testy akceptacyjne przypadku: uruchamiam testy "

    feature = 'docs/features/uruchamiam_testy.feature'

    def step_monitorowane_sa_pliki(self, monitored_files):
        u'monitorowane są pliki "([^"]+)"'

        self.monitored_files = monitored_files

    def step_plik_testow_zawiera_num_testow(self, test_file, tests_num):
        u'plik "([^"]+)" zawiera (.+) testów'

        self.test_file = test_file
        self.tests_num = int(tests_num)
        self.output = StringIO()
        self.controller = TddMon(self.test_file, output=self.output)

    def step_failures_num_testow_skutkuje_niepowodzeniem(self, failures_num):
        u'(.+) testów skutkuje niepowodzeniem'

        self.failures_num = int(failures_num)

    def step_errors_num_testow_skutkuje_bedem(self, errors_num):
        u'(.+) testów skutkuje błędem'

        self.errors_num = int(errors_num)

    def step_coverage_procent_kodu_pokryte_jest_testami(self, coverage):
        u'(.+) procent kodu pokryte jest testami'

        self.coverage = int(coverage)

    def step_zmienie_plik(self, modified_file):
        u'zmienię plik "([^"]+)"'

        self.modified_file = modified_file

    @patch('tddmon.subprocess.Popen')
    def step_wykona_sie_test(self, Popen):
        u'wykona się test'
        if self.failures_num or self.errors_num:
            failures = []
            if self.failures_num:
                failures.append('failures=%d' % self.failures_num)
            if self.errors_num:
                failures.append('errors=%d' % self.errors_num)
            status = 'FAILED (%s)' % (', '.join(failures))
        else:
            status = 'OK'
        stderrdata = 'Ran %d test in 0.038s\n\n%s\n' % (self.tests_num, status)
        test_result = ('', stderrdata)
        stdoutdata = 'TOTAL 1619 750 584 405 %d%%\n' % self.coverage
        coverage_result = (stdoutdata, '')
        Popen().communicate.side_effect = [test_result, coverage_result]

        self.controller.run()

    def step_zobacze_na_czerwono_informacje_o_wykonaniu_testow(self, tests_num):
        u'zobaczę na czerwono informację o wykonaniu (.+) testów'

        color = DEFAULT_COLORS['red']
        expected = '^\033\[%sm\s+%d' % (color, int(tests_num))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_czerwono_informacje_o_failures_num_testach_zakonczonych_niepowodzeniem(self, failures_num):
        u'zobaczę na czerwono informację o (.+) testach zakończonych niepowodzeniem'

        color = DEFAULT_COLORS['red']
        expected = '^\033\[%sm\s+\d+\s+%d' % (color, int(failures_num))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_czerwono_informacje_o_errors_num_testach_zakonczonych_bedem(self, errors_num):
        u'zobaczę na czerwono informację o (.+) testach zakończonych błędem'

        color = DEFAULT_COLORS['red']
        expected = '^\033\[%sm\s+\d+\s+\d+\s+%d' % (color, int(errors_num))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_czerwono_informacje_o_pokryciu_kodu_na_poziomie_coverage(self, coverage):
        u'zobaczę na czerwono informację o pokryciu kodu na poziomie (.+)'

        color = DEFAULT_COLORS['red']
        expected = '^\033\[%sm\s+\d+\s+\d+\s+\d+\s+%d' % (color, int(coverage))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_wszystkie_testy_przechodza(self):
        u'wszystkie testy przechodzą'

        self.failures_num = 0
        self.errors_num = 0
        # color = DEFAULT_COLORS['green']
        # expected = '^\033\[%sm\s+\d+\s+0\s+0' % color
        # result = self.output.getvalue()
        # self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_zielono_informacje_o_wykonaniu_tests_num_testow(self, tests_num):
        u'zobaczę na zielono informację o wykonaniu (.+) testów'

        color = DEFAULT_COLORS['green']
        expected = '^\033\[%sm\s+%d' % (color, int(tests_num))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_zielono_informacje_o_pokryciu_kodu_na_poziomie_coverage(self, coverage):
        u'zobaczę na zielono informację o pokryciu kodu na poziomie (.+)'

        color = DEFAULT_COLORS['green']
        expected = '^\033\[%sm\s+\d+\s+\d+\s+\d+\s+%d' % (color, int(coverage))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_w_kolorze_morskim_informacje_o_pokryciu_kodu_na_poziomie_coverage(self, coverage):
        u'zobaczę w kolorze morskim informację o pokryciu kodu na poziomie (.+)'

        color = DEFAULT_COLORS['sea']
        expected = '^\033\[\d+m\s+\d+\s+\d+\s+\d+\033\[%sm\s*%d' % (color, int(coverage))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_poprzedni_test_zakonczy_sie_sukcesem(self):
        u'poprzedni test zakończył się sukcesem'

        self.controller.status_display._last_run_color = DEFAULT_COLORS['green']

    def step_zobacze_na_niebiesko_informacje_o_wykonaniu_tests_num_testow(self, tests_num):
        u'zobaczę na niebiesko informację o wykonaniu (.+) testów'

        color = DEFAULT_COLORS['blue']
        expected = '^\033\[%sm\s+%d' % (color, int(tests_num))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def step_zobacze_na_niebiesko_informacje_o_pokryciu_kodu_na_poziomie_coverage(self, coverage):
        u'zobaczę na niebiesko informację o pokryciu kodu na poziomie (.+)'

        color = DEFAULT_COLORS['blue']
        expected = '^\033\[%sm\s+\d+\s+\d+\s+\d+\s+%d' % (color, int(coverage))
        result = self.output.getvalue()
        self.assertTrue(bool(re.search(expected, result)))

    def test_run_tests(self):
        " Właściwość: Uruchamiam testy "
        Parser().parse_file(self.feature).evaluate(self)


if __name__ == '__main__':  # pragma: nobranch
    unittest.main()  # pragma nocover



#!/usr/bin/env python
# -*- coding: utf-8 -*-

import types
import unittest

from mock import patch, call
from six.moves import StringIO
from smarttest.decorators import test_type

from tddmon import (main, StatusDisplay, DEFAULT_COLORS, LogWriter,
                    TestRunner, TestResultParser, FileMonitor,
                    FileMonitorTimeoutError, TddMon)


@test_type('unit')
class MainParseTestCase(unittest.TestCase):
    """ Test :py:meth:`main`. """

    @patch('tddmon.__main__.sys.stderr')
    def test_should_raise_exception_if_no_filename(self, stderr):
        """ Scenariusz: brak podanego pliku """
        # Arrange
        # Act
        # Assert
        self.assertRaises(SystemExit, main, [])

    @patch('tddmon.__main__.TddMon')
    def test_should_return_filename_when_filename_given(self, TddMon):
        """ Scenariusz: podany tylko plik """
        # Arrange
        # Act
        filename = 'plik.py'
        main([filename])
        # Assert
        TddMon.assert_called_once_with(filename, log=None)
        TddMon().loop.assert_called_once_with()


@test_type('unit')
class StatusDisplayWriteTestCase(unittest.TestCase):
    """ Test :py:meth:`StatusDisplay.write`. """

    def setUp(self):
        self.output = StringIO()
        self.obj = StatusDisplay(self.output)
        super(StatusDisplayWriteTestCase, self).setUp()

    def test_should_write_in_green_on_successful_run(self):
        """ Scenariusz: testy zakończone sukcesem """
        # Arrange
        # Act
        num, failures, errors, coverage = 1, 0, 0, 100
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['green']
        expected = self._prepare_expected(color, failures, errors, num, coverage)
        self.assertEqual(self.output.getvalue(), expected)

    def test_should_write_in_red_on_failed_run(self):
        """ Scenariusz: testy zakończone porażką """
        # Arrange
        # Act
        num, failures, errors, coverage = 1, 1, 0, 80
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['red']
        expected = self._prepare_expected(color, failures, errors, num, coverage)
        self.assertEqual(self.output.getvalue(), expected)

    def test_should_write_in_green_if_no_tests(self):
        """ Scenariusz: brak testów """
        # Arrange
        # Act
        num, failures, errors, coverage = 0, 0, 0, 0
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['green']
        sea = DEFAULT_COLORS['sea']
        expected = self._prepare_expected(color, failures, errors, num,
                                          coverage, coverage_color=sea)
        self.assertEqual(self.output.getvalue(), expected)

    def test_should_write_in_red_if_error_occure(self):
        """ Scenariusz: wystąpiły błędy """
        # Arrange
        # Act
        num, failures, errors, coverage = 1, 0, 1, 50
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['red']
        expected = self._prepare_expected(color, failures, errors, num, coverage)
        self.assertEqual(self.output.getvalue(), expected)

    def test_should_write_in_green_and_sea_on_successful_run_with_not_fully_covered(self):
        """ Scenariusz: testy zakończone sukcesem ale nie pełne pokrycie"""
        # Arrange
        # Act
        num, failures, errors, coverage = 1, 0, 0, 50
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['green']
        sea = DEFAULT_COLORS['sea']
        expected = self._prepare_expected(color, failures, errors, num,
                                          coverage, coverage_color=sea)
        self.assertEqual(self.output.getvalue(), expected)

    def test_should_write_in_blue_on_repeated_green(self):
        """ Scenariusz: ponowne uruchomienie na zielono """
        # Arrange
        # Act
        num, failures, errors, coverage = 1, 0, 0, 100
        self.obj.write(num, failures, errors, coverage)
        self.obj.write(num, failures, errors, coverage)
        # Assert
        color = DEFAULT_COLORS['green']
        expected = self._prepare_expected(color, failures, errors, num, coverage)
        color = DEFAULT_COLORS['blue']
        expected += self._prepare_expected(color, failures, errors, num, coverage)
        self.assertEqual(self.output.getvalue(), expected)

    def _prepare_expected(self, color, failures, errors, num, coverage, coverage_color=None):
        coverage_color = coverage_color if coverage_color is not None else color
        expected = StatusDisplay.pattern % {
            'color': '\033[%sm' % color,
            'num': num,
            'failures': failures,
            'errors': errors,
            'coverage_color': '\033[%sm' % coverage_color if coverage_color != color else '',
            'coverage': coverage,
            'normal_color': '\033[0m',
        }
        return expected


@test_type('unit')
class StatusDisplayWriteHeaderTestCase(unittest.TestCase):
    """ Test :py:meth:`StatusDisplay.write_header`. """

    def test_should_print_header(self):
        """ Scenariusz: nagłówek """
        # Arrange
        output = StringIO()
        obj = StatusDisplay(output)
        # Act
        obj.write_header()
        # Assert
        expected = 'Tests ran Failures  Errors Coverage\n'
        self.assertEqual(output.getvalue(), expected)


@test_type('unit')
class StatusDisplayWriteLastTestCase(unittest.TestCase):
    """ Test :py:meth:`StatusDisplay.write_last`. """

    def test_should_print_empty_line(self):
        """ Scenariusz: pusta linia """
        # Arrange
        output = StringIO()
        obj = StatusDisplay(output)
        # Act
        obj.write_last()
        # Assert
        expected = '\n'
        self.assertEqual(output.getvalue(), expected)


@test_type('unit')
class LogWriterWriteTestCase(unittest.TestCase):
    """ Test :py:meth:`LogWriter.write`. """

    def test_should_write_traceback_to_file_on_first_run(self):
        """ Scenariusz: pierwszy zapis """
        # Arrange
        output = StringIO()
        obj = LogWriter(output)
        # Act
        traceback = 'some traceback'
        obj.write(traceback)
        # Assert
        self.assertEqual(output.getvalue(), traceback)

    def test_should_not_write_to_file_if_nothing_has_changed(self):
        """ Scenariusz: brak zmiany """
        # Arrange
        output = StringIO()
        obj = LogWriter(output)
        # Act
        traceback = 'some traceback'
        obj.write(traceback)
        obj.write(traceback)
        # Assert
        self.assertEqual(output.getvalue(), traceback)

    def test_should_ignore_run_time_change(self):
        """ Scenariusz: ignorowanie czasu wykonania """
        # Arrange
        output = StringIO()
        obj = LogWriter(output)
        # Act
        first_traceback = '''
.F..
======================================================================
FAIL: test_should_return_buzz_on_5 (__main__.FizzbuzzTestCase)
Scenariusz: liczba buzz
----------------------------------------------------------------------
Traceback (most recent call last):
File "fizzbuzz.py", line 51, in test_should_return_buzz_on_5
    assert False
AssertionError

----------------------------------------------------------------------
Ran 4 tests in 0.001s

FAILED (failures=1)'''
        second_traceback = '''
.F..
======================================================================
FAIL: test_should_return_buzz_on_5 (__main__.FizzbuzzTestCase)
Scenariusz: liczba buzz
----------------------------------------------------------------------
Traceback (most recent call last):
File "fizzbuzz.py", line 51, in test_should_return_buzz_on_5
    assert False
AssertionError

----------------------------------------------------------------------
Ran 4 tests in 0.003s

FAILED (failures=1)'''
        obj.write(first_traceback)
        obj.write(second_traceback)
        # Assert
        self.assertEqual(output.getvalue(), first_traceback)


@test_type('unit')
class TestRunnerRunTestCase(unittest.TestCase):
    """ Test :py:meth:`TestRunner.run`. """

    @patch('subprocess.Popen')
    def test_should_return_program_output_with_coverage_report(self, Popen):
        """ Scenariusz: uruchomienie testu z pokryciem kodu """
        # Arrange
        process = Popen.return_value
        process.communicate.side_effect = [('', 'OK'), ('TOTAL', '')]
        command = 'test_command.py'
        obj = TestRunner(command)
        # Act
        stdoutdata, stderrdata = obj.run()
        # Assert
        self.assertEqual(stderrdata, 'OKTOTAL')
        self.assertEqual(len(Popen.call_args_list), 2)
        self.assertTrue(command in Popen.call_args_list[0][0][0])


@test_type('unit')
class TestResultParserParseTestCase(unittest.TestCase):
    """ Test :py:meth:`TestResultParser.parse`. """

    def test_should_return_0_test_num_on_empty_run(self):
        """ Scenariusz: 1 poprawny test """
        # Arrange
        obj = TestResultParser()
        input_data = ''
        # Act
        result = obj.parse(input_data)
        # Assert
        self.assertEqual(result, (0, 0, 0, 0))

    def test_should_return_1_test_num_on_success(self):
        """ Scenariusz: 1 poprawny test """
        # Arrange
        obj = TestResultParser()
        input_data = '''
Ran 1 tests in 0.001s
OK
'''
        # Act
        result = obj.parse(input_data)
        # Assert
        self.assertEqual(result, (1, 0, 0, 0))

    def test_should_return_2_test_num_on_success(self):
        """ Scenariusz: 2 poprawne testy """
        # Arrange
        obj = TestResultParser()
        input_data = '''
Ran 2 tests in 0.001s
OK
'''
        # Act
        result = obj.parse(input_data)
        # Assert
        self.assertEqual(result, (2, 0, 0, 0))

    def test_should_return_failures_num_on_failure(self):
        """ Scenariusz: testy niepoprawne """
        # Arrange
        obj = TestResultParser()
        input_data = '''
Ran 1 tests in 0.001s
FAILED (failures=1)
'''
        # Act
        result = obj.parse(input_data)
        # Assert
        self.assertEqual(result, (1, 1, 0, 0))

    def test_should_return_coverage_on_success(self):
        """ Scenariusz: wyniki z pokryciem kodu """
        # Arrange
        obj = TestResultParser()
        input_data = '''
Ran 1 tests in 0.001s
OK
Name    Stmts   Miss Branch BrMiss  Cover
-----------------------------------------
leap       26      0      6      0   91%
'''
        # Act
        result = obj.parse(input_data)
        # Assert
        self.assertEqual(result, (1, 0, 0, 91))


@test_type('unit')
class FileMonitorWaitForChangeTestCase(unittest.TestCase):
    """ Test :py:meth:`FileMonitor.wait_for_change`. """

    @patch('tddmon.__main__.time')
    @patch.object(FileMonitor, 'code_has_changed')
    def test_should_raise_exception_on_timeout(self, code_has_changed, time):
        """ Scenariusz: timeout """
        # Arrange
        timeout = 10
        interval = 2
        code_has_changed.side_effect = [False] * (timeout // interval + 1)
        obj = FileMonitor(timeout=timeout, interval=interval)
        # Act
        # Assert
        self.assertRaises(FileMonitorTimeoutError, obj.wait_for_change)

    @patch('tddmon.__main__.time')
    @patch.object(FileMonitor, 'code_has_changed')
    def test_should_return_true_if_code_has_changed(self, code_has_changed, time):
        """ Scenariusz: zmieniony plik """
        # Arrange
        code_has_changed.side_effect = [True]
        obj = FileMonitor()
        # Act
        result = obj.wait_for_change()
        # Assert
        self.assertTrue(result)


@test_type('unit')
class FileMonitorCodeHasChangedTestCase(unittest.TestCase):
    """ Test :py:meth:`FileMonitor.code_has_changed`. """

    @patch.object(FileMonitor, 'file_has_changed')
    @patch.object(FileMonitor, 'get_monitored_files')
    def test_should_return_true_if_code_has_changed(self, get_monitored_files, file_has_changed):
        """ Scenariusz: zmienił się plik """
        # Arrange
        get_monitored_files.side_effect = [['file1.py', 'file2.py']]
        file_has_changed.side_effect = [True]
        obj = FileMonitor()
        # Act
        result = obj.code_has_changed()
        # Assert
        self.assertTrue(result)

    @patch.object(FileMonitor, 'get_monitored_files')
    def test_should_return_false_if_no_files_to_monitor(self, get_monitored_files):
        """ Scenariusz: brak plików """
        # Arrange
        get_monitored_files.side_effect = [[]]
        obj = FileMonitor()
        # Act
        result = obj.code_has_changed()
        # Assert
        self.assertFalse(result)

    @patch.object(FileMonitor, 'file_has_changed')
    @patch.object(FileMonitor, 'get_monitored_files')
    def test_should_return_true_if_some_files_has_changed(self, get_monitored_files, file_has_changed):
        """ Scenariusz: zmienił się plik """
        # Arrange
        get_monitored_files.side_effect = [['file1.py', 'file2.py']]
        file_has_changed.side_effect = [False, True]
        obj = FileMonitor()
        # Act
        result = obj.code_has_changed()
        # Assert
        self.assertTrue(result)


@test_type('unit')
class FileMonitorGetMonitoredFilesTestCase(unittest.TestCase):
    """ Test :py:meth:`FileMonitor.get_monitored_files`. """

    @patch('tddmon.__main__.os.walk')
    def test_should_return_generator_with_files(self, walk):
        """ Scenariusz: generator plików """
        # Arrange
        files_list = [['dir', [], ['file1.py', 'file2.py']]]
        files = ['dir/file1.py', 'dir/file2.py']
        walk.side_effect = [files_list]
        obj = FileMonitor()
        # Act
        result = obj.get_monitored_files()
        # Assert
        self.assertTrue(isinstance(result, types.GeneratorType))
        self.assertEqual(list(result), files)

    @patch('tddmon.__main__.os.walk')
    def test_should_return_filter_out_not_monitored_extensions(self, walk):
        """ Scenariusz: odfiltrowanie niemonitorowanych rozszerzeń """
        # Arrange
        files_list = [['dir', [], ['file1.html', 'file2.py']]]
        files = ['dir/file2.py']
        walk.side_effect = [files_list]
        obj = FileMonitor()
        # Act
        result = obj.get_monitored_files()
        # Assert
        self.assertTrue(isinstance(result, types.GeneratorType))
        self.assertEqual(list(result), files)


@test_type('unit')
class FileMonitorFileHasChagnedTestCase(unittest.TestCase):
    """ Test :py:meth:`FileMonitor.file_has_changed`. """

    def setUp(self):
        self.obj = FileMonitor()
        super(FileMonitorFileHasChagnedTestCase, self).setUp()

    @patch('tddmon.__main__.os.stat')
    def test_should_return_true_if_file_mtime_has_changed(self, stat):
        """ Scenariusz: mtime zmienione """
        # Arrange
        stat().st_mtime = 124
        filename = 'file1.py'
        self.obj._mtimes = {filename: 123}
        # Act
        result = self.obj.file_has_changed(filename)
        # Assert
        self.assertTrue(result)

    @patch('tddmon.__main__.os.stat')
    def test_should_return_false_if_new_file(self, stat):
        """ Scenariusz: mtime zmienione """
        # Arrange
        mtime = 124
        stat().st_mtime = mtime
        filename = 'file1.py'
        # Act
        result = self.obj.file_has_changed(filename)
        # Assert
        self.assertFalse(result)

    @patch('tddmon.__main__.os.stat')
    def test_should_return_false_if_mtime_not_changed(self, stat):
        """ Scenariusz: mtime zmienione """
        # Arrange
        mtime = 124
        stat().st_mtime = mtime
        filename = 'file1.py'
        self.obj._mtimes = {filename: mtime}
        # Act
        result = self.obj.file_has_changed(filename)
        # Assert
        self.assertFalse(result)


@test_type('unit')
class FileMonitorShouldMonitorFileTestCase(unittest.TestCase):
    """ Test :py:meth:`FileMonitor.should_monitor_file`. """

    def test_should_return_false_on_not_monitored_extensions(self):
        """ Scenariusz: niemonitorowane pliki """
        # Arrange
        filename = 'test.py'
        obj = FileMonitor(monitored_extensions=['.html'])
        # Act
        result = obj.should_monitor_file(filename)
        # Assert
        self.assertFalse(result)


@test_type('unit')
class TddMonRunTestCase(unittest.TestCase):
    """ Test :py:meth:`TddMon.run`. """

    @patch('tddmon.__main__.StatusDisplay')
    def test_should_run_tests_and_display_result(self, StatusDisplay):
        """ Scenariusz: pojedyncze uruchomienie """
        # Arrange
        stdoutdata = ''
        stderrdata = ''
        obj = TddMon('test_file.py')
        with patch.object(obj, 'test_runner') as test_runner:
            test_runner.run.return_value = (stdoutdata, stderrdata)
            # Act
            obj.run()
            # Assert
            StatusDisplay().write.assert_called_once_with(0, 0, 0, 0)


@test_type('unit')
class TddMonLoopTestCase(unittest.TestCase):
    """ Test :py:meth:`TddMon.loop`. """

    @patch('tddmon.__main__.FileMonitor')
    @patch('tddmon.__main__.StatusDisplay')
    def test_should_stop_on_keayboard_interrupt(self, StatusDisplay, FileMonitor):
        """ Scenariusz: przerwanie działania """
        # Arrange
        output = StringIO()
        obj = TddMon('test_file.py', output=output)
        StatusDisplay().write_header.return_value = 12
        with patch.object(obj, 'run'):
            FileMonitor().wait_for_change.side_effect = [KeyboardInterrupt]
            # Act
            obj.loop()
            # Assert

    @patch('tddmon.__main__.StatusDisplay')
    @patch('tddmon.__main__.FileMonitor')
    def test_should_print_last_on_timeout(self, FileMonitor, StatusDisplay):
        """ Scenariusz: timeout """
        # Arrange
        obj = TddMon('test_file.py')
        FileMonitor().wait_for_change.side_effect = [FileMonitorTimeoutError, KeyboardInterrupt]
        with patch.object(obj, 'run'):
            # Act
            obj.loop()
        # Assert
        StatusDisplay().write_last.assert_called_once_with()

    @patch('tddmon.__main__.StatusDisplay')
    @patch('tddmon.__main__.FileMonitor')
    def test_should_run_tests(self, FileMonitor, StatusDisplay):
        """ Scenariusz: run """
        # Arrange
        obj = TddMon('test_file.py')
        FileMonitor().wait_for_change.side_effect = [True, KeyboardInterrupt]
        # Act
        with patch.object(TddMon, 'run') as run:
            obj.loop()
            # Assert
            run.assert_has_calls([call(), call()])


if __name__ == '__main__':  # pragma: nobranch
    unittest.main()  # pragma nocover
